#include "internal.h"

SV_constraint *SV_constraint_new(SV_expr *e)
{
	SV_constraint *c = malloc(sizeof(SV_constraint));
	SV_assert_memory(c);
	
	c->simulation = NULL;
	c->id = 0;
	c->expr = NULL;
	c->expr_partials = NULL;
	c->deriv = NULL;
	c->deriv_partials = NULL;
	c->max_iterations = SV_DEFAULT_CONSTRAINT_MAX_ITERATIONS;
	c->tolerance = SV_DEFAULT_CONSTRAINT_TOLERANCE;
	
	SV_constraint_replace_expr(c, e);
	
	return c;
}

void SV_constraint_free(SV_constraint *c)
{
	SV_assert(c->simulation==NULL, "Trying to free a constraint that is attached to a simulation.");
	
	SV_expr_free(c->expr);
	SV_variable_map_expr_free(c->expr_partials);
	SV_expr_free(c->deriv);
	SV_variable_map_expr_free(c->deriv_partials);
	
	free(c);
}

void SV_constraint_replace_expr(SV_constraint *c, SV_expr *e)
{
	SV_assert_sanity(c);
	SV_assert_sanity(e);
	
	if (c->expr)
	{
		SV_expr_free(c->expr);
		SV_variable_map_expr_free(c->expr_partials);
		SV_expr_free(c->deriv);
		SV_variable_map_expr_free(c->deriv_partials);
	}
	
	c->expr = e;
	c->expr_partials = SV_find_partials(c->expr);
	c->deriv = SV_find_derivative(c->expr_partials);
	c->deriv_partials = SV_find_partials(c->deriv);
}

void SV_constraint_set_property(SV_constraint *c, SV_constraint_property prop, void *value)
{
	SV_assert_sanity(c);
	switch(prop)
	{
		case SV_constraint_property_max_iterations:
		{
			int max_iterations = *(int*)value;
			SV_assert(max_iterations>0, "max_iterations must be 1 or greater.");
			c->max_iterations = max_iterations;
			break;
		}
		case SV_constraint_property_tolerance:
		{
			double tolerance = *(double*)value;
			SV_assert(tolerance>0, "tolerance must be greater than 0.");
			c->tolerance = tolerance;
			break;
		}
		default:
			SV_error("Invalid property.");
	}
}

void SV_constraint_get_property(SV_constraint *c, SV_constraint_property prop, void *value)
{
	SV_assert_sanity(c);
	switch(prop)
	{
		case SV_constraint_property_max_iterations:
			*(int*)value = c->max_iterations;
			break;
		case SV_constraint_property_tolerance:
			*(double*)value = c->tolerance;
		default:
			SV_error("Invalid property.");
	}
}
