#include "internal.h"

SV_variable_map *SV_variable_map_new()
{
	SV_variable_map *m = malloc(sizeof(SV_variable_map));
	SV_assert_memory(m);
	m->first = NULL;
	return m;
}

int SV_variable_map_find_or_create_element(SV_variable_map *v, SV_variable *key1, unsigned int key2, SV_variable_map_element **e_out)
{
	SV_assert_sanity(v);
	SV_assert_sanity(key1);
	SV_assert_sanity(e_out);
	
	SV_variable_map_element *e;
	for (e=v->first; e; e=e->next)
	{
		if (e->key1==key1 && e->key2==key2)
		{
			*e_out = e;
			return 1;
		}
	}
	
	e = malloc(sizeof(SV_variable_map_element));
	SV_assert_memory(e);
	
	e->key1 = key1;
	e->key2 = key2;
	
	e->value.expr = NULL;
	e->value.number = 0;
	
	e->next = v->first;
	v->first = e;
	
	*e_out = e;
	return 0;
}

void SV_variable_map_expr_free(SV_variable_map *v)
{
	SV_assert_sanity(v);
	
	SV_variable_map_element *e, *next;
	for (e=v->first; e; e=next)
	{
		next = e->next;
		SV_expr_free(e->value.expr);
		free(e);
	}
	free(v);
}

void SV_variable_map_double_free(SV_variable_map *v)
{
	SV_assert_sanity(v);
	
	SV_variable_map_element *e, *next;
	for (e=v->first; e; e=next)
	{
		next = e->next;
		free(e);
	}
	free(v);
}